/* getsap.c ATM End System Address/Identifier handling */
 
/* Written November 1997 by Francis.Dupont@inria.fr, Mohsen.Souissi@inria.fr */
 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/route.h>

#include <netinet/in.h>
#include <netnatm/natm.h>

#include <sys/time.h>
#include <sys/types.h>


#include "../linux-atm.h"
#include "atm.h"
#include "getsap.h"


/* we assume for the moment there is only one sap per interface */

int mib[] = { CTL_NET, PF_ROUTE, 0, 0, NET_RT_IFLIST, 0 };
int size;
caddr_t buf;
static unsigned char atm_prefix[] = ATM_PREFIX;
struct if_msghdr *ifm;
struct sockaddr_dl *sdl;

caddr_t get_addr_esi(void) {
        caddr_t cp;
        caddr_t esip;

        if (sysctl(mib, 6, NULL, &size, NULL, 0) < 0) {
                perror("sysctl0");
                exit(1);
        }
        if ((buf = malloc(size)) == NULL) {
                fprintf(stderr, "malloc failed\n");
                exit(1);
        }
        if (sysctl(mib, 6, buf, &size, NULL, 0) < 0) {
                perror("sysctl");
                exit(1);
        }

        for (cp = buf; cp < buf + size; cp += ifm->ifm_msglen) {
                ifm = (struct if_msghdr *)cp;
                if ((ifm->ifm_type == RTM_IFINFO) && (esip = ifinfo()))
                        return(esip);
        }
        fprintf(stderr, "not found\n");
        return(NULL);
}

#define ROUNDUP(a) \
        ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
#define ADVANCE(x, n) (x += ROUNDUP((n)->sdl_len))

caddr_t ifinfo(void) {
        caddr_t cp;
        int len, i;

        cp = (caddr_t)(ifm + 1);
        len = ifm->ifm_msglen;

        for (i = 0; i < RTAX_MAX; i++)
                if (ifm->ifm_addrs & (1 << i)) {
                        sdl = (struct sockaddr_dl *)cp;
                        if ((i == RTAX_IFP) && (sdl->sdl_family == AF_LINK))
                                if ((cp = linksock()))
                                        return (cp);
                        ADVANCE(cp, sdl);
                }

        return (NULL);
}

caddr_t linksock(void) {
        caddr_t cp;

        if ((sdl->sdl_nlen == strlen(EN0NAME)) &&
            (strcmp(sdl->sdl_data, EN0NAME) == 0) &&
            (sdl->sdl_alen == 6)) {
                cp = (caddr_t)(sdl->sdl_data + sdl->sdl_nlen);
             return (cp);
        }
        return (NULL);
}


void get_local_sap(struct sap_addr *sap)
{
  caddr_t esip;

  sap->bhli.hl_type = ATM_HL_NONE;
  sap->blli = NULL;
  *sap->pub = 0;
  bcopy(atm_prefix, sap->prv, PREFIX_LEN);
  esip = get_addr_esi();
  bcopy(esip, sap->prv + PREFIX_LEN, ESI_LEN);
  sap->prv[ATM_ESA_LEN -1] = 0x00;   /* SEL zeroed */
}
