/* Some ATM declarations from <linux/ *.h> for compatibility with linux */
 
/* Written 1995,1996 by Werner Almesberger, EPFL LRC */


#ifndef LINUX_ATM_H
#define LINUX_ATM_H

/* stuff from <linux/kernel.h> */
 
/*
 * from atm.h
 */

/* general ATM constants */
#define ATM_CELL_SIZE		    53	/* ATM cell size incl. header */
#define ATM_CELL_PAYLOAD	    48	/* ATM payload size */
#define ATM_MAX_VPI                 255  /* maximum VPI at the UNI */
#define ATM_MAX_VPI_NNI                 4096  /* maximum VPI at the NNI */
#define ATM_MAX_VCI                 65535  /* maximum VCI */

/* "protcol" values for the socket system call */
#define ATM_NO_AAL	0		/* AAL not specified */
#define ATM_AAL0	13		/* "raw" ATM cells */
#define ATM_AAL5	5		/* AAL5 (data) */


/* Traffic description */

#define ATM_NONE	0		/* no traffic */
#define ATM_UBR		1
#define ATM_CBR		2
#define ATM_VBR		3
#define ATM_ABR		4
#define ATM_ANYCLASS	5		/* compatible with everything */

#define ATM_MAX_PCR	-1		/* maximum available PCR */

struct atm_trafprm {
	unsigned char	traffic_class;	/* traffic class (ATM_UBR, ...) */
	int		max_pcr;	/* maximum PCR in cells per second */
	int		min_pcr;	/* minimum PCR in cells per second */
	int		max_cdv;	/* maximum CDV in microseconds */
	int		max_sdu;	/* maximum SDU in bytes */
};

struct atm_qos {
	struct atm_trafprm txtp;	/* parameters in TX direction */
	struct atm_trafprm rxtp;	/* parameters in RX direction */
	unsigned char aal;
};

#define ESI_LEN         6

#define ATM_OC3_PCR	(155520000/270*260/8/53)
			/* OC3 link rate:  155520000 bps
			   SONET overhead: /270*260 (9 section, 1 path)
			   bits per cell:  /8/53
			   max cell rate:  353207.547 cells/sec */

/* SVC addressing */

#define	ATM_ESA_LEN	20		/* ATM End System Address length */
#define ATM_E164_LEN	12		/* maximum E.164 number length */

#define ATM_MAX_BLLI	16		/* maximum number of BLLI elements */

#define ATM_AFI_DCC	0x39		/* DCC ATM Format */
#define ATM_AFI_ICD	0x47		/* ICD ATM Format */
#define ATM_AFI_E164	0x45		/* E.164 ATM Format */


/*
 * SAP structures
 */

#define ATM_MAX_HLI	7	/* maximum high-layer information length */
#define ATM_HL_NONE        0       /* HL not specified */

struct atm_blli {
    unsigned char l2_proto;	/* layer 2 protocol */
    union {
	struct {
	    unsigned char mode;	/* mode of operation (ATM_IMD_xxx), 0 if */
				/* absent */
	    unsigned char window; /* window size (k), 1-127 (0 to omit) */
	} itu;			/* ITU-T encoding */
	unsigned char user;	/* user-specified l2 information */
    } l2;
    unsigned char l3_proto;	/* layer 3 protocol */
    union {
	struct {
	    unsigned char mode;	/* mode of operation (ATM_IMD_xxx), 0 if */
				/* absent */
	    unsigned char def_size; /* default packet size (log2), 4-12 */
	                            /* (0 to omit) */
	    unsigned char window;/* packet window size, 1-127 (0 to omit) */
	} itu;			/* ITU-T ecoding */
	unsigned char user;	/* user specified l3 information */
	struct {		/* if l3_proto = ATM_L3_TR9577 */
	    unsigned char ipi;	/* initial protocol id */
	    unsigned char snap[5]; /* IEEE 802.1 SNAP identifier */
				  /* (only if ipi == NLPID_IEEE802_1_SNAP) */
	} tr9577;
    } l3;
    struct atm_blli *next;	/* next BLLI or NULL (undefined when used in */
				/* atmsvc_msg) */
};


struct atm_bhli {
    unsigned char hl_type;	/* high layer information type */
    unsigned char hl_length;	/* length (only if hl_type == ATM_HL_USER || */
				/* hl_type == ATM_HL_ISO) */
    unsigned char hl_info[ATM_MAX_HLI]; /* high layer information */
};


struct sap_addr {
  unsigned char   prv[ATM_ESA_LEN]; /* private ATM address */
  char            pub[ATM_E164_LEN+1]; /* public address (E.164) */
  /* unused addresses must be bzero'ed */
  struct atm_blli *blli;          /* local SAP, low-layer information */
  struct atm_bhli bhli;           /* local SAP, high-layer information */
};

/* From atmsvc.h, added as_add and as_drop for p2m */

enum atmsvc_msg_type { as_catch_null, as_bind, as_connect, as_add, as_accept, 
		       as_reject, as_listen, as_okay, as_error, as_indicate, as_drop,
		       as_close, as_itf_notify };

/* added to identify the ATM connection configuration */
typedef enum {p2p, root, leaf} CONN_CONF;

struct atmsvc_msg {
    u_int8_t bllis;               /* number of bllis coded on 1 octet */    
    enum atmsvc_msg_type type;
    unsigned long vcc;            /* to identify the SOCKET struct */
    unsigned long listen_vcc;     /* indicate */
    short ep_ref;                 /* < 0 (p2p), = 0 (first party of p2m), */
                                  /* > 0 for next parties (p2m) */
    int reply;                    /* for okay and close:   */
                                  /*   < 0: error before active      */
                                  /* (sigd has discarded ctx) */
                                  /*   ==0: success                  */
                                  /*   > 0: error when active (still */
                                  /*        need to close)           */
    struct sockaddr_natm pvc;     /* indicate, okay (connect) */
    struct sap_addr local;        /* local sap address */
    struct atm_qos qos;           /*  QOS parameters */
    struct sap_addr svc;          /* MUST BE BEFORE "blli", SVC address */
    struct atm_blli blli[1];      /* MUST BE LAST */
};

#define SELECT_TOP_PCR(tp) ((tp).max_pcr && (tp).max_pcr != ATM_MAX_PCR ? \
  (tp).max_pcr : (tp).min_pcr ? (tp).min_pcr : ATM_MAX_PCR)

#endif
