/* active.c - ATM signaling test active client */

/* Written 1997-1998 by Mohsen.Souissi@inria.fr, INRIA */

#include <sys/types.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netnatm/natm.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>

#include "../linux-atm.h"
#include "atmd.h"
#include "atm.h"

#include "inatmarp.h"
#include "ipatmlookup.h"
#include "callroute.h"
#include "io.h"

#define COMPONENT	"ACTIVE"
#define BUF_LEN 40

int debug = 0, inatmarp_dump = 0;
int active;                     /* = 1 if the client is active, 0 otherwise */

const char *sig_qos = NULL;
const char *dump_dir = NULL;


static void menu_display(void) 
{
  system("clear");
  printf("\t\t****************************************\n");
  printf("\t\t*          ATM SIGNALING API           *\n");
  printf("\t\t****************************************\n\n\n");
  printf("1) SHOW Current ATM Calls\n");
  printf("2) SETUP a New ATM Call\n");
  printf("3) ADD an IP Route\n");
  printf("4) DELETE an IP Route\n");
  printf("5) CLOSE an ATM Call (deletes route if any)\n");
  printf("6) CLOSE ALL \n");
  printf("7) ADD a Party to an ATM Call\n");
  printf("8) DROP a Party from an ATM Call \n");
  printf("9) QUIT\n");
  printf("\nchoice: ");
}

static int whatnow(void) 
{
  int num, choice, len;
  char choice_str[BUF_LEN];

  printf("\n\n\n\n");
  printf("1) Back to MENU\n");
  printf("2) QUIT\n");
  printf("\nchoice: ");
  if (fgets(choice_str, sizeof(choice_str), stdin) == NULL) 
    return(whatnow());
  if ((len = strlen(choice_str)) < 2)
    return(whatnow());
  choice_str[len - 1] = '\0';   /* replacing '\n' */
  if ((num = str2num(choice_str)) < 0) {
    printf("\n whatnow: str2num: %s : not a number", choice_str);
    return(whatnow()); 
  }
  else {
    choice = num;
    switch (choice) {
    case 1:
      return(1);
    case 2:
      if (AtmCalls != NULL) {
	printf("\nwhatnow: !!! STILL %d PENDING CALL(S)", NbCalls);
	printf("\n==> GO BACK TO MENU AND CLOSE ALL CALLS\n");
	return(whatnow());
      }
      return(2);
      default:
	printf("\nwhatnow: %d BAD CHOICE\n", choice);
	return(whatnow());
    }
  }
}

static int case2(void)		/* Setup a new ATM Call */
{
  int ret, sub;
  char *ipaddr;
  int num, choice, len;
  char choice_str[BUF_LEN];
  CONN_CONF conf;

  printf("\n\n\n\n");
  printf("1) Point-to-Point ATM Call\n");
  printf("2) Point-to-Multipoint ATM Call\n");
  printf("3) Back to MENU\n");
  printf("\nchoice: ");
  if (fgets(choice_str, sizeof(choice_str), stdin) == NULL) 
    return(case2());
  if ((len = strlen(choice_str)) < 2)
    return(case2());
  choice_str[len - 1] = '\0';   /* replacing '\n' */
  if ((num = str2num(choice_str)) < 0) {
    printf("\n case2: str2num: %s : not a number", choice_str);
    return(case2()); 
  }
  else {
    choice = num;
    switch (choice) {
    case 1:
      conf = p2p;
      break;
    case 2:
      conf = root;
      break;
    case 3:
      return(1);
      default:
	printf("\ncase2: %d BAD CHOICE\n", choice);
	return(case2());
    }
  }
  
  printf("\n Adresse IPATM : ");
  ipaddr = (char *) malloc((IP_LEN + 1)*sizeof(char));
  if (fgets(ipaddr, (IP_LEN + 1)*sizeof(char), stdin) == NULL)
    return(1);
  ipaddr[strlen(ipaddr) - 1] = '\0';  /* replacing '\n' */
  if (badip(ipaddr) != 0) {
    printf("\ncase2: badip: not an IP address\n");
    fflush(stdout);
    (void) sleep(2);
    return(1);
  }
  ret = atmcall_setup(ipaddr, (conf == root) ? 0 : -1);
  if ((sub = whatnow()) == 1)
    return(1);
  return(ret);
}

static int case7(void)		/* Add Party */
{
  int ret, sub, len;
  char *ipaddr;
  char vcc_str[BUF_LEN];
  u_long vcc;
  ATMCALL *call;

  printf("\n\n\n\n");
  printf("\n Adresse IPATM : ");
  ipaddr = (char *) malloc((IP_LEN + 1)*sizeof(char));
  if (fgets(ipaddr, (IP_LEN + 1)*sizeof(char), stdin) == NULL)
    return(1);
  ipaddr[strlen(ipaddr) - 1] = '\0';  /* replacing '\n' */
  if (badip(ipaddr) != 0) {
    printf("\ncase7: badip: not an IP address\n");
    fflush(stdout);
    (void) sleep(2);
    return(1);
  }
  
  printf("\n vcc : ");
  if (fgets(vcc_str, sizeof(vcc_str), stdin) == NULL) 
    return(1);
  len = strlen(vcc_str);
  vcc_str[len - 1] = '\0';         /* replacing '\n' */
  if ( (len < 2) || ((ret = str2num(vcc_str)) < 0) ) {
    printf("\ncase7: str2num: %s : not a number", vcc_str);
    fflush(stdout);
    (void) sleep(2);
    return(1);
  }
  vcc = (u_long) ret;
  if ((call = atmcall_lookup(vcc)) == NULL) {
    printf("case7: no matching atm call for VCC %ld\n", vcc);
    return(1);
  }
  if (call->conf != root) {
    printf("case7: ATM call is not in p2m root conf\n");
    return(1);
  }
  ret = active_add_party(call, ipaddr);
  if ((sub = whatnow()) == 1)
    return(1);
  return(ret);
}

static int case8(void)		/* Drop Party */
{
  int ret, sub, len;
  char vcc_str[BUF_LEN], epr_str[BUF_LEN];
  u_long vcc;
  u_short ep_ref;

  printf("\n\n\n\n");
  
  printf("\n vcc : ");
  if (fgets(vcc_str, sizeof(vcc_str), stdin) == NULL) 
    return(1);
  len = strlen(vcc_str);
  vcc_str[len - 1] = '\0';         /* replacing '\n' */
  if ( (len < 2) || ((ret = str2num(vcc_str)) < 0) ) {
    printf("\ncase8: str2num: %s : not a number", vcc_str);
    fflush(stdout);
    (void) sleep(2);
    return(1);
  }
  vcc = (u_long) ret;

  printf("\n ep_ref : ");
  if (fgets(epr_str, sizeof(epr_str), stdin) == NULL) 
    return(1);
  len = strlen(epr_str);
  epr_str[len - 1] = '\0';         /* replacing '\n' */
  if ( (len < 2) || ((ret = str2num(epr_str)) < 0) ) {
    printf("\ncase8: str2num: %s : not a number", epr_str);
    fflush(stdout);
    (void) sleep(2);
    return(1);
  }
  ep_ref = (u_short) ret;

  ret = active_drop_party(vcc, ep_ref, NULL);
  if ((sub = whatnow()) == 1)
    return(1);
  return(ret);
}


static int main_loop(void)
{
  int choice, ret = 0, fin = 0, sub = 0, len;
  char choice_str[BUF_LEN], vcc_str[BUF_LEN];
  u_long vcc;
  ATMCALL *call;


  while(!fin) {
  menu:
    /*      if (fcntl(sig_sd, F_SETFL, O_NONBLOCK) == -1) {
	    perror("fcntl");
	    return(-1);
	    } 
	    recv_atmsigd(NULL); */
    menu_display();
    if (fgets(choice_str, sizeof(choice_str), stdin) == NULL)
      goto menu;
    if ((len = strlen(choice_str)) < 2) 
      goto menu;
    choice_str[len - 1] = '\0';   /* replacing '\n' */
    if ((ret = str2num(choice_str)) < 0) {
      printf("\nmain_loop: str2num: %s : Not a number\n", choice_str);
      fflush(stdout);
      (void) sleep(2);
      goto menu;
    }
    else {
      choice = ret;
      switch (choice) {
      case 1:
	show_atmcalls();
	if ((sub = whatnow()) == 1)
	  goto menu;
	fin = 1;
	break;
      case 2:
	if ((ret = case2()) == 1)
	  goto menu;
	fin = 1;
	break;
      case 3:
      case 4:
      case 5:
	printf("\n vcc : ");
	if (fgets(vcc_str, sizeof(vcc_str), stdin) == NULL) 
	  goto menu;
	len = strlen(vcc_str);
	vcc_str[len - 1] = '\0';         /* replacing '\n' */
	if ( (len < 2) || ((ret = str2num(vcc_str)) < 0) ) {
	  printf("\nmain_loop: str2num: %s : not a number", vcc_str);
	  fflush(stdout);
	  (void) sleep(2);
	  goto menu;
	}
	vcc = (u_long) ret;
	switch (choice) {
	case 3:
	  ret = iponatm(AAREQUEST, vcc, inatmarp_dump);
	  break;
	case 4:
	  ret = (call =  delete_ipatmroute(vcc, NULL)) ? 0 : -1;
	  break;
	case 5:
	  ret = active_close_atmcall(0, vcc, NULL);
	}
	if ((sub = whatnow()) == 1)
	  goto menu;
	fin = 1;
	break;
      case 6:
        ret = close_all(0);
	if ((sub = whatnow()) == 1)
          goto menu;
	fin = 1;
        break;
      case 7:
	if ((ret = case7()) == 1)
	  goto menu;
	fin = 1;
	break;
      case 8:
	if ((ret = case8()) == 1)
	  goto menu;
	fin = 1;
	break;
      case 9: 
	if (AtmCalls != NULL) {
	  printf("\nmain_loop: !!! STILL %d PENDING CALL(S)", NbCalls);
	  printf("\n==> GO BACK TO MENU AND CLOSE ALL CALLS\n");
	  (void) sleep(2);
	  goto menu;
	}
	fin = 1;
	break;
      default:
	printf("\nmain_loop: %d BAD CHOICE\n", choice);
	fflush(stdout);
	(void) sleep(2);
      }	
    }
  }
  return(ret);
}
/* ------------------------------- main   ------------------------------- */
  
int main(void)
{
  int ret = 0;

  active = 1;
  inatmarp_dump = 1;
  set_application("ACTIVE");
  dump_dir = NULL;
  set_verbosity(NULL, DIAG_DEBUG);
  set_verbosity("QMSG", DIAG_INFO);
  diag(COMPONENT, DIAG_INFO, "\nATM signaling active client "
#ifdef UNI30
      "UNI 3.0"
#endif
#ifdef UNI31
       "UNI 3.1"
#ifdef ALLOW_UNI30
       "+3.0compat"
#endif
#endif
       "/AAL5, version " VERSION "\n");
  
  if ((ret = open_atmsigd()) != 0) 
    return(ret);
  init_current_time();
  init_addr();
  CurrVcc = 0;
  ret = main_loop();
  close(sig_sd);
  return(ret);
}
