/* io.c - I/O operations */
 
/* Written 1997-1998 by Mohsen.Souissi@inria.fr, INRIA */
 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/route.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <netnatm/natm.h>

#include <sys/time.h>
#include <sys/types.h>
#include <netdb.h>

#include "../linux-atm.h"
#include "atm.h"
#include "atmsap.h"
#include "atmd.h"
#include "getsap.h"
#include "inatmarp.h"
#include "ipatmlookup.h"
#include "callroute.h"
#include "io.h"
/* #include "../qgen/uni.h" */

#define COMPONENT "IO"


#define SERV_IP "0.0.0.0"
#define ATMSIG_PORT 6666


struct timeval now;
int sig_sd;

/* for atm qos_default_p2[pm] */
struct atm_qos qos_default_p2p = {{ATM_UBR, 0, 0, 0, 0},
				  {ATM_UBR, 0, 0, 0, 0},
				  ATM_AAL5};

struct atm_qos qos_default_p2m = {{ATM_UBR, 0, 0, 0, 0},
				  {ATM_NONE, 0, 0, 0, 0},
				  ATM_AAL5};

/* ---------------------------- addresses ------------------------------*/
/* we assume for the moment there is only one sap per interface */

struct sap_addr local_sap[MAX_SAP];

struct sap_addr *get_sap(u_char *aesa, u_char sel)
{
  struct sap_addr *sap;
  
  sap = alloc(sizeof(struct sap_addr)+sizeof(struct atm_blli));
  sap->bhli.hl_type = ATM_HL_NONE;
  sap->blli = NULL;
  *sap->pub = 0;
  bcopy(aesa, sap->prv, PREFIX_LEN + ESI_LEN);
  sap->prv[ATM_ESA_LEN -1] = sel;   /* SEL zeroed */
  return(sap);
}

void init_addr(void)
{
  int i, j;

  get_local_sap(&local_sap[0]);

  for (i = 1; i < MAX_SAP; i++) {
  bcopy(&(local_sap[0].prv[0]), &local_sap[i], PREFIX_LEN + ESI_LEN);
  local_sap[i].prv[PREFIX_LEN + ESI_LEN] = 0x00 + (u_char)i;
  printf("\nLOCAL_SAP[%d] = ", i);
  for (j=0; j < PREFIX_LEN + ESI_LEN + 1; j++)
    printf("%02X", local_sap[i].prv[j]);
  printf("\n");
  }
}

/* ----------------------------- signaling messages -------------------- */

const char *as_name[] = { "<invalid>", "as_bind", "as_connect", "as_add",
			  "as_accept", "as_reject", "as_listen","as_okay", 
			  "as_error", "as_indicate", "as_drop", "as_close" };



/* -------------------- Interface with atmsigd ------------------------- */

/* open a TCP socket to communicate with the signaling daemon (atmsigd) */
int open_atmsigd(void)   
{
    int s;
    struct sockaddr_in server_in;

    if ((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
	perror("socket");
	return (1);
    }
    memset(&server_in, 0, sizeof(server_in));
    server_in.sin_len = sizeof(server_in);
    server_in.sin_family = AF_INET;
    server_in.sin_addr.s_addr = inet_addr(SERV_IP);
    server_in.sin_port =  htons(ATMSIG_PORT);
    if (connect(s, (struct sockaddr *) &server_in, sizeof(server_in)) < 0) {
	perror("connect");
	return (1);
    }
    /* intializing sig_sd */
    sig_sd = s;
    return (0);
}


#define SCLIENT_BASE_LEN (sizeof(struct atmsvc_msg)-sizeof(struct atm_blli))
#define SCLIENT_BUFFER_SIZE (sizeof(struct atmsvc_msg)+(ATM_MAX_BLLI-1)* \
  sizeof(struct atm_blli)+1)


int recv_atmsigd(ATMCALL *callarg, PARTY *partyarg)
{
    static long abuffer[SCLIENT_BUFFER_SIZE / sizeof(long) + 1];
    u_char *buffer = (u_char *)abuffer;
    static u_char header;
    int cc, size, msg_len, ret;
    u_int8_t bllis;

/* first do a peek to know if there is a msg to extract */
    if ((cc = recv(sig_sd, &header, 1, MSG_PEEK)) < 0) {
      printf("ERROR:  recv atmsigd: %s\n", strerror(errno));
      return(-1);	
    }
    if (cc == 0) {
      printf("ERROR: recv atmsigd: tcp_socket (%d) closed by peer\n", sig_sd);
      return(-2);
    }
    bllis = (u_int) header;
    msg_len = SCLIENT_BASE_LEN+bllis*sizeof(struct atm_blli);
    if (bllis > ATM_MAX_BLLI) {
      printf("FATAL: atmsigd message too big (%d >= %d)\n",
	   msg_len, SCLIENT_BUFFER_SIZE);
      return(-3);
    }
    if ((size = read(sig_sd, buffer, msg_len)) < 0) {
      printf("ERROR:  read atmsigd: %s\n", strerror(errno));
      return(-4);
    }
    if (size < SCLIENT_BASE_LEN) {
      printf("FATAL: atmsigd message too short (%d < %d)",
	   size, SCLIENT_BASE_LEN);
      return(-5);
    }
    if (size != msg_len) {
      printf("FATAL: atmsigd message has bad length (got %d, base is %d, "
	   "incr is %d)\n", size, SCLIENT_BASE_LEN, sizeof(struct atm_blli));
      return(-6);
    }
    ret = from_atmsigd((struct atmsvc_msg *) buffer, callarg, partyarg);
    if (ret < 0)
      return(-7);
    return(ret);
}

int send_atmsigd(u_long vcc, u_long listen_vcc, short ep_ref,  
		  enum atmsvc_msg_type type, int reply, 
		  const struct sap_addr *svc, const struct sap_addr *local, 
		  const struct atm_qos *qos)
{
    struct atmsvc_msg *msg;
    struct atm_blli *walk;
    int  msg_len, i, ret;
    u_int8_t bllis;

    bllis = 0;
    for (walk = svc ? svc->blli : NULL; walk; walk = walk->next)
	bllis++;
    msg_len = bllis2msg(bllis);
    msg = alloc(msg_len);
    msg->bllis = bllis;
    msg->vcc = vcc;  
    msg->listen_vcc = listen_vcc;
    msg->ep_ref = ep_ref;
    msg->type = type;
    msg->reply = reply;
    memset(&msg->pvc, 0, sizeof(msg->pvc));
    if (qos != NULL) 
      msg->qos = *qos;
    else 
      memset(&msg->qos, 0, sizeof(msg->qos));
    if (local == NULL)
	memset(&msg->local, 0, sizeof(msg->local));
    else {
	msg->local = *local;
	msg->local.blli = NULL;
    }
    if (svc == NULL) 
      memset(&msg->svc, 0, sizeof(msg->svc));
    else {
	msg->svc = *svc;
	i = 0;
	for (walk = svc->blli; walk; walk = walk->next)
	    msg->blli[i++] = *walk;
    }
    ret = to_atmsigd(msg);
    free(msg);
    return(ret);
}

int to_atmsigd(struct atmsvc_msg *msg)
{
    int size, wrote;
   
    size = bllis2msg(msg->bllis);
    printf("TO ATMSIGD: %s, <reply %d>, <vcc %ld, listen_vcc %ld>, "
	   "<length %d>\n", as_name[msg->type], msg->reply, msg->vcc, 
	   msg->listen_vcc, size);
    if ((wrote = write(sig_sd, msg, size)) == size) 
	return(0);
    if (wrote < 0) {
	fprintf(stderr, "to_atmsigd write: %s", strerror(errno));
	if (errno == EPIPE)
	  printf("ERROR:  atmsigd is down\n");
	return(wrote);
    }
    printf("ERROR: bad atmsigd write: wanted %d, wrote %d\n",
	 size, wrote);
    return(wrote);
}

/* -------------------- Native ATM communication ------------------------- */

int recv_natm_peer(ATMCALL *call, int dump)
{
  int ret;

  if ((ret = iponatm(AAREPLY, call->vcc, dump)) != 0) {
    printf("recv_natm_peer: iponatm failed\n");
    return(ret);
  }
  return(0);
}


/* ----- common part ------------------------------------------------------- */

void init_current_time(void)
{
    gettimeofday(&now,NULL);
}

/* polling messages from atmsigd */
int poll_atmsigd(ATMCALL *callarg, PARTY *partyarg)               
{
  fd_set set;
  int fds, ret;
  
  FD_ZERO(&set);
  FD_SET(sig_sd, &set);
  fds = FD_SETSIZE;
    gettimeofday(&now, NULL);
    ret = select(fds, &set, NULL, NULL, next_timer());
    if (ret < 0) {
      if (errno != EINTR) 
	perror("select");
      return(-1);
    }
    printf("\tWaiting for incoming messages.....\n");
    gettimeofday(&now, NULL);
    if (FD_ISSET(sig_sd, &set)) 
      if ((ret = recv_atmsigd(callarg, partyarg)) < 0) {
	if (ret == -2)	     /* sigd has closed its tcp socket with client */
	  close(sig_sd);
	return(-2);
      }
    /* expire timers after handling messges to make sure we don't
       time out unnecessarily because of scheduling delays */
    expire_timers();
    return(ret);
}


/* polling for messages from atmsigd and from peer clients,
   used by passive client */
int poll_loop(void)               
{
  fd_set perm, set;
  int fds, ret;
  ATMCALL *call;
  int sig_down = 0;

  FD_ZERO(&perm);
  FD_SET(sig_sd, &perm);
  fds = FD_SETSIZE;
  gettimeofday(&now, NULL);
  while (1) {
    set = perm;
    
    for (call = AtmCalls; call; call =  call->next)
      if (call->sd > 0) 
	FD_SET(call->sd, &set);
    ret = select(fds, &set, NULL, NULL, next_timer());
    if (ret < 0) {
      if (errno != EINTR) 
	perror("select");
      return(-1);
    }
    else {
      printf("----------");
      gettimeofday(&now, NULL);
      
      if (FD_ISSET(sig_sd, &set)) 
	if ((ret = recv_atmsigd(NULL, NULL)) < 0) {
	  if (ret == -2) {     /* sigd has closed his tcp socket with client */
	    sig_down = 1;
	    printf("\n\npoll_loop: !!! CONNECTION WITH ATMSIGD IS DOWN !!!\n");
	    printf("TEARING DOWN CURRENT ATM CALLS\n");
	    (void) close_all(sig_down);
	    close(sig_sd);
	  }
	  return(-2);
	}

      for (call = AtmCalls; call; call =  call->next)
	if((call->sd > 0) && (FD_ISSET(call->sd, &set)))
	  if ((ret =  recv_natm_peer(call, inatmarp_dump)) != 0) {
	    printf("recv_natm_peer: failed to read message from peer\n");
	    return(-1);
	  }

      /* expire timers after handling messges to make sure we don't
	 time out unnecessarily because of scheduling delays */
      expire_timers();
    }
  }
}
