/* passive.c - ATM signaling test passive client */

/* Written 11-1997 by Mohsen.Souissi@inria.fr, INRIA */

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netnatm/natm.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>

#include "../linux-atm.h"
#include "atm.h"
#include "atmd.h"

#include "inatmarp.h"
#include "ipatmlookup.h"
#include "callroute.h"
#include "io.h"

#define COMPONENT	"PASSIVE"
#define MAX_CON 1

int debug = 0, inatmarp_dump = 0;
int active;              /* = 1 if the client is active, 0 otherwise */

const char *sig_qos = NULL;
const char *dump_dir = NULL;


static int init_listen(void)
{
  int  ret;

  ret = send_atmsigd(0, 0, -1, as_listen, 0, 
		     NULL, &local_sap[0], &qos_default_p2p);
  if (ret != 0) {
    printf("ERROR: init_listen: failed to send as_listen\n");
    return(ret);
  }
  if ((ret = poll_atmsigd(NULL, NULL)) != 0) {
    printf("ERROR: init_listen: got trouble when receiving from atmsigd\n");
    return(ret);
  }
  return(0);
}
  
  
/* ------------------------------- main ...  ------------------------------- */

int main(void)
{
  int ret;

  active = 0;
  inatmarp_dump = 1;
  set_application("PASSIVE");
  dump_dir = NULL;
  set_verbosity(NULL, DIAG_DEBUG);
  diag(COMPONENT, DIAG_INFO, "\nATM signaling passive client "
#ifdef UNI30
       "UNI 3.0"
#endif
#ifdef UNI31
       "UNI 3.1"
#ifdef ALLOW_UNI30
       "+3.0compat"
#endif
#endif
       "/AAL5, version " VERSION "\n");
  
  if ((ret = open_atmsigd()) != 0) {
    printf("ERROR: main: failed to open atmsigd\n");
    return(ret);
  }
  init_current_time();
  init_addr();
  CurrVcc = 0;
  if ((ret = init_listen()) != 0) {
    printf("ERROR: main: failed to init listen\n");
    return(ret);
  }
  ret = poll_loop();
  if (ret != 0) {
    ret = close_all(0);
    if (ret != 0)
      printf("ERROR: main: got trouble when closing all\n");
  }
  close(sig_sd);
  return (ret);
}
