/* uni.c - Processing of incoming UNI v3.x (Q.2931/Q.2971) messages */
 
/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
/* Modified 1997-1998 by Mohsen.Souissi@inria.fr, INRIA */
 
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netnatm/natm.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

#include "../linux-atm.h"
#include "atm.h"
#include "atmd.h"
#include "uni.h"
#include "qlib.h"

#include "proto.h"
#include "sap.h"
#include "io.h"
#include "timeout.h"
#include "trace.h"

/* #define __KERNEL__ */ /* since that's what we effectively are */ /* see after...*/


#define COMPONENT "UNI"


extern const char *cause_text[]; /* from mess.c */


static Q_DSC in_dsc;
static TIMER *t309 = NULL;


static int send_call_proceeding(SOCKET *sock, PARTY *party)
{
    Q_DSC dsc;
    int size;

    q_create(&dsc, q_buffer, MAX_Q_MSG);
    q_assign(&dsc, QF_msg_type, QMSG_CALL_PROC);
    q_assign(&dsc, QF_call_ref, sock->call_ref);
    if ((net) && (party == NULL)) { /* only in p2p conf */
	int vci;

	vci = get_vci();
	if (vci < 0) {
	    (void) q_close(&dsc);
	    return(vci);
	}
	q_assign(&dsc, QF_vpi, 0); /* @@@ */
	q_assign(&dsc, QF_vci, vci);
	sprintf(sock->pvc.snatm_if, "en0");
	sock->pvc.snatm_vpi = 0;
	sock->pvc.snatm_vci = vci;
    }
    if (party != NULL)
    q_assign(&dsc, QF_ep_ref, party->ep_ref | 0x8000);
    if ((size = q_close(&dsc)) >= 0) 
      to_signaling(q_buffer, size);
    return(0);
}


static void setup_call(u_long call_ref, short ep_ref)
{
    SOCKET *sock, *this, **walk;
    SCLIENT *scl;
    struct sap_addr *sap, *new_sap;
    struct atm_qos *qos, *new_qos;
    u_char buf[ATM_ESA_LEN];
    int len, i;
    unsigned long num;
    PARTY *party = NULL;

    if (ep_ref <= 0) {		/* p2p or first leaf conf */
      qos = alloc(sizeof(struct atm_qos));
      new_qos = alloc(sizeof(struct atm_qos));
    }
    else {
      new_qos = qos = NULL;
    }
    sap = sap_decode(&in_dsc, qos);   /* p2p/leaf conf */
    new_sap = NULL;
    sock = lookup_sap(sap, qos, &new_sap, new_qos); /* later: @@@ */
    if (!sock || new_sap) 
      free(sap);
    else 
      new_sap = sap; /* @@@ should strip extra BLLIs */
    if (new_qos != NULL)
      new_qos->aal = ATM_AAL5; /* hack @@@ */
    if (!sock) {
	send_release_complete(call_ref, ATM_CV_INCOMP_DEST); /* @@@ dunno */
	return;
    }
    scl = sock->sclient;
    if (scl->state == DYING) {    /* later: ?????? */  
	if (sock->state == ss_wait_rel) 
	    STOP_TIMER(sock);
	free_sock(sock);
	return;
    }
    if ((num = scl->connections->num + 1) == 0xffffff) 
      num = 1;
    this = new_sock(scl, num, (ep_ref < 0) ? p2p : leaf);
    this->state = ss_indicated;
    this->call_state = cs_in_proc;
    this->call_ref = call_ref;
    if (ep_ref >= 0) {
      party = new_party(this, (u_short) ep_ref, ps_add_recv);
      this->nbactp = 1;
    }

#ifdef CISCO
    else
#endif
    {
	int error;

	error = send_call_proceeding(this, party);
	if (error) {
	    free_sock(this);
	    send_release_complete(call_ref, ATM_CV_NO_CI);
	    return;
	}
    }
    this->local = new_sap;
    if (ep_ref < 0)
      this->qos = *new_qos;
    else
      bzero(&this->qos, sizeof(struct atm_qos));
    /* if (sock->local) *this->local->sas_addr = sock->local->sas_addr; ??? */
    diag(COMPONENT, DIAG_DEBUG, "AAL type %ld", q_fetch(&in_dsc, QF_aal_type));
    len = q_read(&in_dsc, QF_cdpn_esa, (void *) &buf, sizeof(buf));
    if (debug) {
	printf("Addr len %d:", len);
	for (i = 0; i < (len < 30 ? len : 30); i++) printf(" %02X",buf[i]);
	putchar('\n');
    }
    if (net == 0) {             /* already set by send_call_proceeding */
      sprintf(this->pvc.snatm_if, EN0NAME);
      this->pvc.snatm_vpi = q_fetch(&in_dsc, QF_vpi);
      this->pvc.snatm_vci = q_fetch(&in_dsc, QF_vci);
    }
    diag(COMPONENT,DIAG_DEBUG,"<CR = 0x%06X> ITF.VPI.VCI: %s.%d.%d", call_ref, 
	 this->pvc.snatm_if, this->pvc.snatm_vpi, this->pvc.snatm_vci);
    if (ep_ref >= 0)
      diag(COMPONENT, DIAG_DEBUG, "<EPR = %d>", party->ep_ref);      
    this->remote = alloc_t(struct sap_addr);
    memset(this->remote, 0, sizeof(struct sap_addr));
    if (q_present(&in_dsc, QF_cgpn)) {       /* should handle E.164 too */
      char buffer[MAX_ATM_ADDR_LEN+1]; 
      
      i = q_read(&in_dsc, QF_cgpn, (void *) this->remote->prv,
		 ATM_ESA_LEN);  /* later: modify atm2text @@@ */
      if (atm2text_sap(buffer, MAX_ATM_ADDR_LEN+1, this->remote, pretty) < 0) 
	strcpy(buffer, "<invalid address>");
	diag(COMPONENT,DIAG_DEBUG, "Incoming call from %s", buffer); 
    }
    if (ep_ref >= 0)
      party->remote = sap_copy(this->remote);
    send_sclient(scl, 0, num, ep_ref, as_indicate, 0, &this->pvc,
		 this->remote, NULL, new_qos);   
    for (walk = &sock->listen; *walk; walk = &(*walk)->listen)
      continue;
    *walk = this;
    diag(COMPONENT, DIAG_DEBUG, "SE vpi.vci=%d.%d", this->pvc.snatm_vpi,
	 this->pvc.snatm_vci);         /* conversion pb see later */
}


static void send_status(SOCKET *sock, u_long call_ref, PARTY *party, 
			u_char cause,...)
{
    va_list ap;
    Q_DSC dsc;
    int size;

    q_create(&dsc, q_buffer, MAX_Q_MSG);
    q_assign(&dsc, QF_msg_type, QMSG_STATUS);
    if (sock) {
	q_assign(&dsc, QF_call_ref, sock->call_ref);
	q_assign(&dsc, QF_call_state, (int) sock->call_state);
	if (party != NULL) {	/* @@@ see after */
	    q_assign(&dsc, QF_ep_ref,
		     party->ep_ref | (sock->conf == root ? 0 : 0x8000));
	    q_assign(&dsc, QF_ep_state, party->pstate);
	}
    }
    else {			/* see later for ep_ref @@@ */
	q_assign(&dsc, QF_call_ref, call_ref);
	q_assign(&dsc, QF_call_state, 0); /* U0 - Null / REST 0 - Null */
    }
    q_assign(&dsc, QF_cause, cause);
    va_start(ap, cause);
    switch (cause) {
	case ATM_CV_UNKNOWN_MSG_TYPE:
	case ATM_CV_INCOMP_MSG:
	    q_assign(&dsc, QF_bad_msg_type, va_arg(ap, unsigned char));
	    break;
	case ATM_CV_MAND_IE_MISSING:
	case ATM_CV_INVALID_IE:
	    {
		unsigned char ie;

		ie = va_arg(ap, unsigned char);
		q_write(&dsc, QF_ie_id6, &ie, 1);
		break;
	    }
	default:
	    ;
    }
    va_end(ap);
    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer, size);
}


static void send_status_enq(SOCKET *sock, PARTY *party)
{
    Q_DSC dsc;
    int size;

    q_create(&dsc, q_buffer, MAX_Q_MSG);
    q_assign(&dsc, QF_msg_type, QMSG_STATUS_ENQ);
    q_assign(&dsc, QF_call_ref, sock->call_ref);
    if (party != NULL)
      q_assign(&dsc, QF_ep_ref,
	       party->ep_ref | (sock->conf == root ? 0 : 0x8000));
    if ((size = q_close(&dsc)) >= 0) 
      to_signaling(q_buffer, size);
    /* see later @@@ should start T322 */
}


static void send_connect_ack(SOCKET *sock)
{
    Q_DSC dsc;
    int size;

    q_create(&dsc, q_buffer,MAX_Q_MSG);
    q_assign(&dsc, QF_msg_type,QMSG_CONN_ACK);
    q_assign(&dsc, QF_call_ref, sock->call_ref);
    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer, size);
}


static void send_restart_ack(int vpi, int vci)
{
    Q_DSC dsc;
    int size;

    q_create(&dsc,q_buffer,MAX_Q_MSG);
    q_assign(&dsc,QF_msg_type,QMSG_REST_ACK);
    q_assign(&dsc,QF_call_ref,0);
    if (!vpi && !vci) q_assign(&dsc, QF_rst_class, ATM_RST_ALL_VC);
    else {			/* it's ATM_RST_ALL_VC in the original code! */
	q_assign(&dsc, QF_rst_class, ATM_RST_IND_VC);
	q_assign(&dsc, QF_vpi, vpi);
	q_assign(&dsc, QF_vci, vci);
    }
    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer, size);
}

static void process_uni(u_char mid, SOCKET *sock, PARTY *party)
{
  /*  char buffer[MAX_ATM_ADDR_LEN+1]; */
  int error;
  u_char cause;
      
  if (sock->sclient->state == DYING) {  
    diag(COMPONENT, DIAG_WARN, 
	 "sclient %d (DYING) is receiving a message %d",
	 sock->sclient->id, mid);
    if (sock->state == ss_wait_rel) 
      STOP_TIMER(sock);
    free_sock(sock);
    return;
  }
  switch (mid) {
  case QMSG_STATUS: /* 5.5.6.12 */
    {
      CALL_STATE state;
      PARTY_STATE pstate;
      
      /*
       * NOTE: T322 isn't implemented yet, but when it is, make sure
       *	 to only stop it on STATUS iff the cause is
       *	 ATM_CV_RESP_STAT_ENQ. Supplementary services break if
       *	 you stop on any STATUS.
       */
      if ((sock->call_state == cs_rel_req) ||
	  (sock->call_state == cs_rel_ind))
	return;
      if (party == NULL) {
	state = q_fetch(&in_dsc,QF_call_state);
	if (state == cs_null)
	  break; /* clear call */
	if (state != sock->call_state)
	  diag(COMPONENT, DIAG_WARN, "STATUS %s received in state %s",
	       cs_name[state], cs_name[sock->call_state]);
      }
      else {
	pstate = q_fetch(&in_dsc, QF_ep_state);
	if (pstate == ps_null)
	  break; /* clear call */
	if (pstate != party->pstate)
	  diag(COMPONENT, DIAG_WARN, "STATUS %s received in state %s",
	       ps_name[pstate], ps_name[party->pstate]);
	
      }
    }
  return;
  default:
    ;
  }

  switch (mid) {		/* yet another switch block */

  case QMSG_CALL_PROC: /* CONNECTING, WAIT_REL, REL_REQ */
    if (sock->state == ss_wait_rel || sock->state == ss_rel_req) {
      send_status(sock, 0, NULL, ATM_CV_INCOMP_MSG, QMSG_CALL_PROC);
      return;
    }
    if (sock->state != ss_connecting) break;
    /* check for 2nd CALL_PROC @@@ */
    STOP_TIMER(sock);
    if (q_present(&in_dsc, QG_conn_id)) {      /* see later @@@ */
      sprintf(sock->pvc.snatm_if, "en0");
      sock->pvc.snatm_vpi = q_fetch(&in_dsc, QF_vpi);
      sock->pvc.snatm_vci = q_fetch(&in_dsc, QF_vci);
      diag(COMPONENT, DIAG_DEBUG, "ITF.VPI.VCI: %s.%d.%d",
	   sock->pvc.snatm_if, sock->pvc.snatm_vpi,
	   sock->pvc.snatm_vci);
    }
    START_TIMER(sock, T310);
    sock->call_state = cs_out_proc;
    return;

  case QMSG_CONNECT: /* CONNECTING, REL_REQ, p2p or first party of p2m */
    if (sock->state == ss_rel_req) {
      send_status(sock, 0, NULL, ATM_CV_INCOMP_MSG, QMSG_CONNECT);
      return;
    }
    if (sock->state != ss_connecting)
      break;
    STOP_TIMER(sock);
    if (q_present(&in_dsc, QG_conn_id)) {
      /* see later */
      sprintf(sock->pvc.snatm_if, "en0");
      sock->pvc.snatm_vpi = q_fetch(&in_dsc, QF_vpi);
      sock->pvc.snatm_vci = q_fetch(&in_dsc, QF_vci);
      diag(COMPONENT, DIAG_DEBUG, "ITF.VPI.VCI: %s.%d.%d",
	   sock->pvc.snatm_if, sock->pvc.snatm_vpi,
	   sock->pvc.snatm_vci);
    }
    error = 0;
    if (((sock->pvc.snatm_vpi == 0) && (sock->pvc.snatm_vci) == 0) ||
	((party != NULL) && (party->ep_ref != 0))) {
      error = -ENOPROTOOPT;
      /* more problems */
      if (error) {
	set_error(sock, NULL, error);
	if ((party != NULL) && (party->ep_ref != 0))   /* 1st party in p2m */
	  diag(COMPONENT, DIAG_WARN,
	       "<EPR = %d> bad value (must be = 0)", party->ep_ref); 
	/* @@@ cause follows reason ??? */
	send_release(sock->call_ref, 0); 
	START_TIMER(sock, T308_1);
	new_state(sock, ss_rel_req);
	return;
      }
    }
    send_connect_ack(sock);
    /* @@@ fill in sock->remote */
    /* @@@ fill in traffic parameters */  /* see later */
    send_sclient(sock->sclient, sock->num, 0, party ? party->ep_ref : -1, 
		 as_connect, 0, &sock->pvc, NULL, sock->local, &sock->qos);
    new_state(sock, ss_connected);
    if (party != NULL) {
      party->pstate = ps_active;
      diag(COMPONENT, DIAG_INFO, 
	   "Active open succeeded (<CR = 0x%06X>, party [%d, %ld, %d])", 
	   sock->call_ref, sock->sclient->id, sock->num, party->ep_ref);
    }
    else {
      /* see later the cast op below */
      /*	    if (atm2text(buffer, MAX_ATM_ADDR_LEN+1, 
		    (struct sockaddr *) sock->local, 0) < 0) 
		    strcpy(buffer, "<invalid>"); */
      diag(COMPONENT, DIAG_INFO, 
	   "Active open succeeded (<CR = 0x%06X>, sock [%d, %ld])", 
	   sock->call_ref, sock->sclient->id, sock->num);
    }
    return;
    
  case QMSG_ADD_PARTY_ACK:
    if (party != NULL)
      switch (party->pstate) {
      case ps_add_init:
	STOP_TIMER(party);
	send_sclient(sock->sclient, sock->num, 0, party->ep_ref, as_add, 
		     0, NULL, NULL, NULL, NULL);
	party->pstate = ps_active;
	return;
      default:
	diag(COMPONENT, DIAG_WARN,
	     "UNI message %s is incompatible with ep_state %s", 
	     mid2name(mid), ps_name[party->pstate]);
	return;
      }
    else
      diag(COMPONENT, DIAG_WARN, 
	   "No matching party for UNI message %s", mid2name(mid));
    return;
    
  case QMSG_DROP_PARTY:
    cause = q_fetch(&in_dsc, QF_cause);
    diag(COMPONENT, DIAG_DEBUG, "Cause %d (%s)", cause, 
	 cause > 127 ? "invalid cause" : cause_text[cause]);
    
    if (party != NULL) {
      if (sock->state != ss_connected)
	break;
      switch (party->pstate) {
      case ps_active:
	party->pstate = ps_drop_recv;
	send_drop_sdu(party);
	if (--sock->nbactp == 0) {
	  free_party(party);
	  send_close_sdu(sock);
	  /* Active close (as if) */
	  diag(COMPONENT,DIAG_INFO,"Active close <CR = 0x%06X>",
	       sock->call_ref);
	  send_release(sock->call_ref,
#ifdef UNI31
		       ATM_CV_NORMAL_CLEAR
#else
		       ATM_CV_NORMAL_UNSPEC
#endif
		       );
	  START_TIMER(sock,T308_1);
	  new_state(sock, ss_rel_req);
	  return;
	}
	else {
	  send_drop_party_ack(sock->call_ref, party->ep_ref, cause);
	  send_drop_sdu(party);
	}
	return;
      default:
	diag(COMPONENT, DIAG_WARN,
	     "UNI message %s is incompatible with ep_state %s", 
	     mid2name(mid), ps_name[party->pstate]);
	return;
      }
    }

  case QMSG_ADD_PARTY_REJ:
    cause = q_fetch(&in_dsc, QF_cause);
    diag(COMPONENT, DIAG_DEBUG, "Cause %d (%s)", cause, 
	 cause > 127 ? "invalid cause" : cause_text[cause]);
    if (party != NULL)
      switch (party->pstate) {
      case ps_add_init:
	set_error(sock, party, -ECONNREFUSED);
	send_drop_party_ack(sock->call_ref, party->ep_ref, cause); 
	party->pstate = ps_drop_recv;
	STOP_TIMER(party);
	send_drop_sdu(party);
	return;
      default:
	diag(COMPONENT, DIAG_WARN,
	     "UNI message %s is incompatible with ep_state %s", 
	     mid2name(mid), ps_name[party->pstate]);
	return;
      }

  case QMSG_CONN_ACK: /* ACCEPTING, WAIT_REL, REL_REQ, p2p or leaf of p2m */
    if (party != NULL)
      diag(COMPONENT, DIAG_DEBUG, "CA <CR, EPR> =<0x%06X, %d>",
	   sock->call_ref, party->ep_ref);
    else
      diag(COMPONENT, DIAG_DEBUG, "CA vpi.vci=%d.%d",
	   sock->pvc.snatm_vpi, sock->pvc.snatm_vci);
    if (sock->state == ss_wait_rel || sock->state == ss_rel_req) {
      send_status(sock, 0, party, ATM_CV_INCOMP_MSG, QMSG_CONN_ACK);
      return;
    }
    if (sock->state != ss_accepting) 
      break;
    if (party != NULL)
      if (party->pstate != ps_add_recv)
	break;
    STOP_TIMER(sock);
    send_sclient(sock->sclient, sock->num, 0, party ? party->ep_ref : -1, 
		 as_accept, 0, NULL, NULL, sock->local, NULL);
    new_state(sock, ss_connected);
    if (party != NULL) {
      party->pstate = ps_active;
      /*   if (atm2text(buffer, MAX_ATM_ADDR_LEN+1, 
	   (struct sockaddr *) sock->local, 0) < 0) 
	   strcpy(buffer, "<invalid>"); */    /* see later */
      diag(COMPONENT, DIAG_INFO, 
	   "Passive open party succeeded <CR = 0x%06X> [%d, %ld, %d])", 
	   sock->call_ref, sock->sclient->id, sock->num, party->ep_ref);
    }
    else
      diag(COMPONENT, DIAG_INFO, 
	   "Passive open succeeded <CR = 0x%06X>, sock [%d, %ld])", 
	   sock->call_ref, sock->sclient->id, sock->num);
    return;
    
  case QMSG_RELEASE: /* all states */
    cause = q_fetch(&in_dsc, QF_cause);
    diag(COMPONENT, DIAG_DEBUG, "Cause %d (%s)", cause, 
	 cause > 127 ? "invalid cause" : cause_text[cause]);
    if (sock->parties != NULL)	/* dropping all parties if any */
      free_all_parties(sock);
    switch (sock->state) {
    case ss_connecting:		/* p2p or root */
      set_error(sock, NULL, -ECONNREFUSED);
      /* fall through */
    case ss_accepting:		/* p2p or leaf */
      set_error(sock, NULL, -ECONNRESET); /* ERESTARTSYS ? */
      send_release_complete(sock->call_ref, 0);
      /* fall through */
    case ss_rel_req:		/* p2p, root or leaf */
      send_close_sdu(sock);
      /* fall through */
    case ss_wait_rel:		/* p2p, root or leaf */
      STOP_TIMER(sock);
      free_sock(sock);
      return;
    case ss_connected:
      diag(COMPONENT, DIAG_INFO, 
	   "Passive close on <CR = 0x%06X>, sock [%d, %ld])",
	   sock->call_ref, sock->sclient->id, sock->num);
      send_close_sdu(sock);
      new_state(sock, ss_rel_ind);
      return;
    case ss_indicated:
      send_release_complete(sock->call_ref, 0);
      new_state(sock, ss_zombie);
      /* fall through */
    case ss_rel_ind:
      /* send_release_complete(sock->call_ref, 0); */
      return;
    default:
      send_release_complete(sock->call_ref, 0); /* @@@ should
						   be ATM_CV_INCOMP_MSG */
      break;
    }
    break;
    
  case QMSG_DROP_PARTY_ACK:
    if (q_present(&in_dsc,QF_cause) == 0)
      cause = 0;
    else {
      cause = q_fetch(&in_dsc, QF_cause);
      diag(COMPONENT, DIAG_DEBUG, "Cause %d (%s)", cause,
	   cause > 127 ? "invalid cause" : cause_text[cause]);
    }
    switch (party->pstate) {
    case ps_drop_init:
      send_drop_sdu(party);
      STOP_TIMER(party);
      free_party(party);
      return;
    default:
      diag(COMPONENT, DIAG_WARN,
	   "UNI message %s is incompatible with ep_state %s", 
	   mid2name(mid), ps_name[party->pstate]);
      return;
    } /* (party == NULL) has already been tested in first_pass_uni */

  case QMSG_RESTART:
    set_error(sock, NULL, -ENETRESET);
    /* fall through */
  case QMSG_STATUS: /* fall through when clearing */
    if (party != NULL) {
      if (q_present(&in_dsc,QF_cause) == 0)
	cause = 0;
      else {
	cause = q_fetch(&in_dsc, QF_cause);
	diag(COMPONENT, DIAG_DEBUG, "Cause %d (%s)", cause,
	     cause > 127 ? "invalid cause" : cause_text[cause]);
      }
      switch (party->pstate) {
      case ps_add_init:
      case ps_add_recv:
	set_error(sock, party, -ECONNRESET); /* ERESTARTSYS ? */
	/* fall through */
      case ps_drop_recv:
	send_drop_sdu(party);
	/* fall through */
      case ps_drop_init:
	STOP_TIMER(party);
	free_party(party);
	return;
      case ps_active:
	diag(COMPONENT, DIAG_INFO, 
	     "Passive close_party on <CR = 0x%06X>, party [%d, %ld, %d])",
	     sock->call_ref, sock->sclient->id, sock->num, party->ep_ref);
	send_drop_sdu(party);
	return;
      default:
	break;
      }
      break;
    }

  case QMSG_REL_COMP: /* basically any state (except LISTENING and
			 ZOMBIE) */
    if (mid != QMSG_REL_COMP || !q_present(&in_dsc,QF_cause))
      cause = 0;
    else {
      cause = q_fetch(&in_dsc, QF_cause);
      diag(COMPONENT, DIAG_DEBUG, "Cause %d (%s)", cause,
	   cause > 127 ? "invalid cause" : cause_text[cause]);
    }
    switch (sock->state) {
    case ss_connecting:
      set_error(sock, NULL, cause == ATM_CV_UNALLOC ?
		-EADDRNOTAVAIL : cause == ATM_CV_RES_UNAVAIL ||
#ifdef UNI31
		cause == ATM_CV_UCR_UNAVAIL ||
#endif
		cause == ATM_CV_NO_ROUTE_DEST ? -EHOSTUNREACH :
		cause == ATM_CV_NUM_CHANGED ? -ECONNABORTED :
		cause == ATM_CV_DEST_OOO ? -EHOSTDOWN :
		-ECONNREFUSED);
      /* fall through */
    case ss_accepting:
      set_error(sock, NULL, -ECONNRESET); /* ERESTARTSYS ? */
      /* fall through */
    case ss_rel_req:
      send_close_sdu(sock);
      /* fall through */
    case ss_wait_rel:
      STOP_TIMER(sock);
      free_sock(sock);
      return;
    case ss_connected:
      diag(COMPONENT, DIAG_INFO, 
	   "Passive close on <CR = 0x%06X>, sock [%d, %ld])",
	   sock->call_ref, sock->sclient->id, sock->num);
      send_close_sdu(sock);
      /* fall through */
    case ss_rel_ind:
      new_state(sock, ss_wait_close);
      return;
    case ss_indicated:
      new_state(sock, ss_zombie);
      return;
    default:
      break;
    }
    break; /* fail */

  default:
    diag(COMPONENT,DIAG_WARN, "Bad UNI message %d", mid);
    send_status(sock, 0, NULL, ATM_CV_UNKNOWN_MSG_TYPE, mid);
    return;
  } /* see this warning later @@@ */
  diag(COMPONENT, DIAG_WARN,
       "UNI message %s is incompatible with state %s/%s %s %s",
       mid2name(mid), state_name[sock->state], cs_name[sock->call_state],
       party ? "(ep_state = )" : "", party ? ps_name[party->pstate] : "");
  send_status(sock, 0, party, ATM_CV_INCOMP_MSG, mid);
}


static void clear_all_calls(void)
{
    SOCKET *curr,*next;
    
    for (curr = sockets; curr; curr = next) {
      next = curr->next;
      if (curr->call_state != cs_null) 
	process_uni(QMSG_RESTART, curr, NULL);
    }
}


void clear_all_calls_on_T309(void)
{
    clear_all_calls();
    t309 = NULL;
}


void saal_failure(void)
{
    SOCKET *curr,*next;

    trace_msg("SAAL went down");
    for (curr = sockets; curr; curr = next) {
      next = curr->next;
      if (curr->call_state != cs_null)
	if (curr->call_state != cs_active)
	  process_uni(QMSG_RESTART, curr, NULL);
	else 
	  if (!t309) t309 = start_timer(T309_TIME, on_T309, NULL);
    }
}


void saal_okay(void)
{
  SOCKET *curr;
  PARTY *party;
  static saal_init = 0;
  
  trace_msg("SAAL came up");
  /* FD: force a restart when saal comes OK the first time! */
  if (!saal_init) {
    Q_DSC dsc;
    int size;
    
    saal_init++;
    q_create(&dsc,q_buffer,MAX_Q_MSG);
    q_assign(&dsc,QF_msg_type,QMSG_RESTART);
    q_assign(&dsc,QF_call_ref,0);
    q_assign(&dsc, QF_rst_class, ATM_RST_ALL_VC);
    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer, size);
  }
  if (!t309) return;
  stop_timer(t309);
  t309 = NULL;
  for (curr = sockets; curr; curr = curr->next)
    if (curr->call_state != cs_null) { 
      send_status_enq(curr, NULL);
      if (curr->parties != NULL)        /* if p2m call */
	for (party = curr->parties; party != NULL; party = party->next)
	  if (party->pstate != ps_null)
	    send_status_enq(curr, party);
    }
}


static void first_pass_uni(void *msg)
{
  SOCKET *curr;
  PARTY *party = NULL;
  unsigned long call_ref;
  unsigned char mid;
  short ep_ref;
  
  call_ref = q_fetch(&in_dsc, QF_call_ref)^0x800000;
  mid = q_fetch(&in_dsc, QF_msg_type);
  if (mid == QMSG_REST_ACK) return;
  if (mid == QMSG_RESTART) { /* 5.5.5.2 */
    int rst_class;
    
    rst_class = q_fetch(&in_dsc, QF_rst_class);
    switch (rst_class) {
    case ATM_RST_IND_VC:
      {
	int vpi, vci;
	
	if (!q_present(&in_dsc, QG_conn_id)) {
	  send_status(NULL, 0L, NULL, ATM_CV_MAND_IE_MISSING,
		      ATM_IE_CONN_ID);
	  return;
	}             /* @@@ see later */
	vpi = q_fetch(&in_dsc, QF_vpi);
	vci = q_fetch(&in_dsc, QF_vci);
	for (curr = sockets; curr; curr = curr->next)
	  if (curr->pvc.snatm_vpi == vpi &&
	      curr->pvc.snatm_vci == vci) break;
	if (!curr) {
	  send_status(NULL, 0L, NULL, ATM_CV_INVALID_IE, ATM_IE_CONN_ID);
	  return;
	}
	process_uni(mid, curr, NULL);
	send_restart_ack(vpi, vci);
      }
    break;
    case ATM_RST_ALL_VC:
      clear_all_calls();
      send_restart_ack(0,0);
      break;
    default:
      send_status(NULL, 0L, NULL, ATM_CV_INVALID_IE, ATM_IE_RESTART);
    }
    return;
  }
  if (!(call_ref & 0x7fffff)) {
    return; /* bad things happen ... @@@ */
  }
  
  for (curr = sockets; curr; curr = curr->next)
    if (curr->call_ref == call_ref) 
      break;
  /* Call reference procedural errors 5.6.5.3.1 */
  if ((curr == NULL) || (curr->call_state == cs_null))
    switch(mid) {
    case QMSG_ADD_PARTY:
    case QMSG_ADD_PARTY_ACK:
    case QMSG_ADD_PARTY_REJ:
    case QMSG_DROP_PARTY:
    case QMSG_DROP_PARTY_ACK:
      diag(COMPONENT, DIAG_WARN, 
	   "No matching call for UNI message %s <CR = 0x%06X>", 
	   mid2name(mid), call_ref);
      send_release_complete(call_ref, ATM_CV_INV_CR);
      return;
    default:
      ;
    }

  /* check if it is a p2m message */
  if (q_present(&in_dsc, QF_ep_ref) != 0)
    ep_ref = q_fetch(&in_dsc, QF_ep_ref) & 0x7fff;
  else
    ep_ref = -1;

  if (ep_ref >= 0) {
    if (curr != NULL) {
      for (party = curr->parties; party != NULL; party = party->next)
	if (party->ep_ref == (u_short) ep_ref)
	  break;
      if (party == NULL) {	/* EPR procedural errors 5.6.5.3.2.2 */
	if ((mid != QMSG_SETUP) && (mid != QMSG_ADD_PARTY) 
	    && (mid != QMSG_DROP_PARTY_ACK)) {
	  send_drop_party_ack(call_ref, (u_short) ep_ref, 
			      ATM_CV_INV_EPR); /* cause 89 */
	  return;	
	}
	if (mid == QMSG_DROP_PARTY_ACK)
	  return;		/* just ignore it */
	diag(COMPONENT, DIAG_WARN, 
	     "No matching party for <CR, EPR> = <0x%06X, %d>", 
	     call_ref, ep_ref);
	return;
      }
      else
	diag(COMPONENT, DIAG_DEBUG, 
	     "FROM NET: %s (0x%02X) <CR = 0x%06X> for party [%d, %ld, %d]",
	     mid2name(((u_char *) msg)[5]), ((u_char *)msg)[5], 
	     call_ref, curr->sclient->id, curr->num, ep_ref);
    }
    else
      diag(COMPONENT, DIAG_DEBUG, 
	   "FROM NET: %s (0x%02X) <CR = 0x%06X> for a new party <EPR = %d>",
	   mid2name(((u_char *) msg)[5]), ((u_char *)msg)[5], call_ref, 
	   ep_ref);
  }
  else {
    if (curr != NULL)
      diag(COMPONENT, DIAG_DEBUG,
	   "FROM NET: %s (0x%02X) <CR = 0x%06X> for sock [%d, %ld]",
	   mid2name(((u_char *) msg)[5]), ((u_char *)msg)[5], call_ref, 
	   curr->sclient->id, curr->num);
    else
      diag(COMPONENT, DIAG_DEBUG, 
	   "FROM NET: %s (0x%02X) <CR = 0x%06X> for a new sock",
	   mid2name(((u_char *) msg)[5]), ((u_char *)msg)[5], call_ref);
  }
  
  if (mid == QMSG_SETUP) {
    if (curr == NULL) 
      setup_call(call_ref, ep_ref);
    return;
  }
  if (mid == QMSG_STATUS_ENQ) {
    send_status(curr, call_ref, party, ATM_CV_RESP_STAT_ENQ);
    return;
  }

  /* if (mid == QMSG_STATUS) */
/*     if (q_fetch(&in_dsc, QF_call_state) != (int) cs_null) { */
/*       send_release_complete(call_ref, ATM_CV_INCOMP_MSG); */
/*       return; */
/*     } */ 

  process_uni(mid, curr, party);
}


static void abort_call(void)
{
    SOCKET *curr;
    unsigned long call_ref;

    diag(COMPONENT,DIAG_ERROR,"can't parse message - aborting the call");
    call_ref = q_fetch(&in_dsc, QF_call_ref)^0x800000;
	/* hope that at least the call ref was okay ... */
    for (curr = sockets; curr; curr = curr->next)
        if (curr->call_ref == call_ref) {
	    process_uni(QMSG_RESTART, curr, NULL);
	    break;
	}
    send_release_complete(call_ref, ATM_CV_PROTOCOL_ERROR);
}


void to_uni(void *msg, int size)
{
    if (q_open(&in_dsc, msg, size) < 0) {
	abort_call();
	return;
    }
    first_pass_uni(msg);
    if (q_close(&in_dsc) < 0)
	diag(COMPONENT, DIAG_ERROR, "q_close returned <0 in to_uni");
}
